@@ -161,7 +161,7 @@ QString checkUpdates::latestVersion( const QByteArray& data )
 		return true ;
 	} ;
 
-	SirikaliJson json( data,SirikaliJson::type::CONTENTS ) ;
+	SirikaliJson json( data,[]( const QString& e ){ utility::debug() << e ; } ) ;
 
 	for( auto& it : json.getTags( "tag_name" ) ){
 
