@@ -208,6 +208,38 @@ public:
 	protected:
 		class commandOptions ;
 	public:
+		struct unMount{
+
+			const QString& cipherFolder ;
+			const QString& mountPoint ;
+			const QString& fileSystem ;
+			int numberOfAttempts ;
+		} ;
+
+		struct exe_args{
+			exe_args( const QString& e,const QStringList& s ) :
+				exe( e ),args( s )
+			{
+			}
+			exe_args()
+			{
+			}
+			QString exe ;
+			QStringList args ;
+		};
+
+		struct exe_args_const{
+			exe_args_const( const exe_args& e ) : exe( e.exe ),args( e.args )
+			{
+			}
+			exe_args_const( const QString& e,const QStringList& s ) :
+				exe( e ),args( s )
+			{
+			}
+			const QString& exe ;
+			const QStringList& args ;
+		};
+
 		class Wrapper{
 		public:
 			Wrapper( const engines::engine& e ) :
@@ -268,7 +300,7 @@ public:
 
 			cryfsMigrateFileSystem,
 			cryfsReplaceFileSystem,
-		        cryfsVersionTooOldToMigrateVolume,
+			cryfsVersionTooOldToMigrateVolume,
 			notSupportedMountPointFolderPath,
 			mountPointFolderNotEmpty,
 			IllegalPath,
@@ -345,9 +377,9 @@ public:
 				createOptions() ;
 
 				QString idleTimeOut ;
-				QString createOpts ;
 				QString configFile ;
 				QString keyFile ;
+				QStringList createOpts ;
 				engines::engine::booleanOptions opts ;
 				bool success = true ;
 			} ;
@@ -376,12 +408,16 @@ public:
 					      const QString& configFile,
 					      const QString& mountOpes,
 					      const QString& keyFile,
+				              const QString& identityFile,
+				              const QString& identityAgent,
 					      const engines::engine::booleanOptions& opts ) ;
 				mountOptions() ;
 				QString idleTimeOut ;
 				QString configFile ;
-				QString mountOpts ;
 				QString keyFile ;
+				QString identityFile ;
+				QString identityAgent ;
+				QStringList mountOpts ;
 				engines::engine::booleanOptions opts ;
 				bool success = true ;
 			} ;
@@ -410,38 +446,31 @@ public:
 
 		struct cmdArgsList
 		{
-			struct options
-			{
-				options( const favorites::entry& e,
-					 const QByteArray& volumeKey = QByteArray() ) ;
+			cmdArgsList( const favorites::entry& e,
+				     const QByteArray& volumeKey = QByteArray() ) ;
 
-				options( const QString& cipher_folder,
-					 const QString& plain_folder,
-					 const QByteArray& volume_key,
-					 const engines::engine::createGUIOptions::createOptions& ) ;
+			cmdArgsList( const QString& cipher_folder,
+				     const QString& plain_folder,
+				     const QByteArray& volume_key,
+				     const engines::engine::createGUIOptions::createOptions& ) ;
 
-				options( const QString& cipher_folder,
-					 const QString& plain_folder,
-					 const QByteArray& volume_key,
-					 const engines::engine::mountGUIOptions::mountOptions& ) ;
+			cmdArgsList( const QString& cipher_folder,
+				     const QString& plain_folder,
+				     const QByteArray& volume_key,
+				     const engines::engine::mountGUIOptions::mountOptions& ) ;
 
-				QString cipherFolder ;
-				QString plainFolder ;
-				QByteArray key ;
-				QString idleTimeout ;
-				QString configFilePath ;
-				QString mountOptions ;
-				QString createOptions ;
-				QString keyFile ;
-				booleanOptions boolOptions ;
-			} ;
-
-			const QString& exe ;
-			const engines::engine::cmdArgsList::options& opt ;
-			const QString& configFilePath ;
-			const QString& cipherFolder ;
-			const QString& mountPoint ;
-			const bool create ;
+			QString cipherFolder ;
+			QString mountPoint ;
+			QByteArray key ;
+			QString idleTimeout ;
+			QString configFilePath ;
+			QString identityFile ;
+			QString identityAgent ;
+			QStringList mountOptions ;
+			QStringList createOptions ;
+			QString keyFile ;
+			QString fuseOptionsSeparator ;
+			booleanOptions boolOptions ;
 		} ;
 
 		struct BaseOptions
@@ -454,7 +483,7 @@ public:
 				int argumentLine ;
 			};
 
-			std::vector< vInfo >versionInfo ;
+			std::vector< vInfo > versionInfo ;
 			int  backendTimeout ;
 			bool hasConfigFile ;
 			bool setsCipherPath ;
@@ -467,20 +496,34 @@ public:
 			bool autorefreshOnMountUnMount ;
 			bool backendRequireMountPath ;
 			bool takesTooLongToUnlock ;
+			bool backendRunsInBackGround ;
+			bool acceptsSubType ;
+			bool acceptsVolName ;
+			bool likeSsh ;
+			bool autoCreatesMountPoint ;
+			bool autoDeletesMountPoint ;
+			bool usesOnlyMountPoint ;
 
 			QByteArray passwordFormat ;
+			QString displayName ;
 			QString releaseURL ;
-			QString minimumVersion ;
+			QString versionMinimum ;
 			QString reverseString ;
 			QString idleString ;
 			QString executableName ;
 			QString incorrectPasswordText ;
 			QString incorrectPassWordCode ;
 			QString configFileArgument ;
-			QString unMountCommand ;
-			QString windowsUnMountCommand ;
+			QString keyFileArgument ;
 			QString windowsInstallPathRegistryKey ;
 			QString windowsInstallPathRegistryValue ;
+			QString windowsExecutableFolderPath ;
+			QString mountControlStructure ;
+			QString createControlStructure ;
+			QString sshOptions ;
+
+			QStringList windowsUnMountCommand ;
+			QStringList unMountCommand ;
 			QStringList volumePropertiesCommands ;
 			QStringList successfulMountedList ;
 			QStringList failedToMountList ;
@@ -495,17 +538,21 @@ public:
 		struct args
 		{
 			args() ;
-			args( const cmdArgsList&,const engines::engine::commandOptions&,
-			      const QString& cmd = QString() ) ;
+			args( const cmdArgsList&,
+			      const engines::engine::commandOptions&,
+			      const QString& cmd,
+			      const QStringList& ) ;
 			QString cmd ;
 			QString cipherPath ;
 			QString mountPath ;
-			QString fuseOptions ;
-			QString exeOptions ;
 			QString mode ;
 			QString subtype ;
+			QStringList cmd_args ;
+			QStringList fuseOptions ;
 		} ;
 
+		static QString encodeMountPath( const QString& e ) ;
+
 		static void encodeSpecialCharacters( QString& ) ;
 		static void decodeSpecialCharacters( QString& ) ;
 
@@ -523,18 +570,31 @@ public:
 		bool customBackend() const ;
 		bool autorefreshOnMountUnMount() const ;
 		bool backendRequireMountPath() const ;
+		bool backendRunsInBackGround() const ;
+		bool acceptsSubType() const ;
+		bool acceptsVolName() const ;
+		bool likeSsh() const ;
+		bool autoCreatesMountPoint() const ;
+		bool autoDeletesMountPoint() const ;
+		bool requiresAPassword() const ;
+		bool requiresNoPassword() const ;
+		bool usesOnlyMountPoint() const ;
 
 		engines::engine::status notFoundCode() const ;
 
 		int backendTimeout() const ;
+
 		const QStringList& names() const ;
 		const QStringList& fuseNames() const ;
 		const QStringList& configFileNames() const ;
 		const QStringList& fileExtensions() const ;
 		const QStringList& volumePropertiesCommands() const ;
+		const QStringList& unMountCommand() const ;
+		const QStringList& windowsUnMountCommand() const ;
 
 		const engines::version& installedVersion() const ;
 
+		const QString& sshOptions() const ;
 		const QString& executableFullPath() const ;
 		const QString& minimumVersion() const ;
 		const QString& reverseString() const ;
@@ -542,39 +602,41 @@ public:
 		const QString& releaseURL() const ;
 		const QString& executableName() const ;
 		const QString& name() const ;
+		const QString& displayName() const ;
+		const QString& uiName() const ;
 		const QString& configFileName() const ;
+		const QString& keyFileArgument() const ;
+		const QString& mountControlStructure() const ;
+		const QString& createControlStructure() const ;
 		const QString& incorrectPasswordText() const ;
 		const QString& incorrectPasswordCode() const ;
-		const QString& unMountCommand() const ;
-		const QString& windowsUnMountCommand() const ;
+		const QString& configFileArgument() const ;
 		const QString& windowsInstallPathRegistryKey() const ;
 		const QString& windowsInstallPathRegistryValue() const ;
+		const QString& windowsExecutableFolderPath() const ;
 
-		engine::engine::error errorCode( const QString& ) const ;
-
-		QString setConfigFilePath( const QString& ) const ;
 		QByteArray setPassword( const QByteArray& ) const ;
 
 		engine( BaseOptions ) ;
 
 		virtual ~engine() ;
 
-		virtual void updateVolumeList( const engines::engine::cmdArgsList::options& ) const ;
+		virtual engine::engine::error errorCode( const QString& ) const ;
 
-		virtual QStringList mountInfo( const QStringList& ) const ;
+		virtual void updateVolumeList( const engines::engine::cmdArgsList& ) const ;
+
+		virtual volumeInfo::List mountInfo( const volumeInfo::List& ) const ;
 
 		virtual Task::future< QString >& volumeProperties( const QString& cipherFolder,
 								   const QString& mountPoint ) const ;
 
-		virtual engines::engine::status unmount( const QString& cipherFolder,
-							 const QString& mountPoint,
-							 int maxCount ) const ;
+		virtual engines::engine::status unmount( const engines::engine::unMount& ) const ;
 
-		virtual bool requiresAPassword( const engines::engine::cmdArgsList::options& ) const ;
+		virtual bool requiresAPassword( const engines::engine::cmdArgsList& ) const ;
 
 		virtual bool takesTooLongToUnlock() const ;
 
-		virtual engine::engine::status passAllRequirenments( const engines::engine::cmdArgsList::options& ) const ;
+		virtual engine::engine::status passAllRequirenments( const engines::engine::cmdArgsList& ) const ;
 
 		struct ownsCipherFolder{
 			bool yes ;
@@ -585,58 +647,49 @@ public:
 		virtual ownsCipherFolder ownsCipherPath( const QString& cipherPath,
 		                                         const QString& configPath ) const ;
 
-		virtual void updateOptions( engines::engine::cmdArgsList::options&,bool ) const ;
+		virtual void updateOptions( engines::engine::commandOptions& opts,
+					    const engines::engine::cmdArgsList& args,
+					    bool creating ) const ;
+
+		virtual void updateOptions( engines::engine::cmdArgsList&,bool creating ) const ;
 
 		virtual const QProcessEnvironment& getProcessEnvironment() const ;
-		virtual bool requiresPolkit() const ;		
+
+		virtual bool requiresPolkit() const ;
+
 		virtual args command( const QByteArray& password,
-				      const engines::engine::cmdArgsList& args ) const ;
+				      const engines::engine::cmdArgsList& args,
+				      bool create ) const ;
+
 		virtual engines::engine::status errorCode( const QString& e,int s ) const ;
 
 		virtual void GUICreateOptions( const createGUIOptions& ) const ;
 
 		virtual void GUIMountOptions( const mountGUIOptions& ) const ;
 	protected:
+		bool unmountVolume( const engine::engine::exe_args_const& exe,bool usePolkit ) const ;
+
 		class commandOptions{
 		public:
-			enum class separator{ space,equal_sign } ;
+			class fuseOptions ;
 
 			class Options{
 			public:
-				Options( QString& m,QString s ) :
-					m_options( m ),m_separator( std::move( s ) )
+				Options( QStringList& m ) : m_options( m )
 				{
 				}
-				bool doesNotContain( const QString& key ) const
-				{
-					return !this->contains( key ) ;
-				}
-				bool contains( const QString& key ) const
-				{
-					return m_options.contains( key ) ;
-				}
-				void addPair( const QString& key,const QString& value,separator s )
-				{
-					if( s == separator::space ){
-
-						this->add( key + " " + value ) ;
-					}else{
-						this->add( key + "=" + value ) ;
-					}
-				}
 				template< typename E >
 				void add( E&& e )
 				{
-					while( m_options.endsWith( "\n" ) ){
-
-						m_options = utility::removeLast( m_options,1 ) ;
-					}
-					if( m_options.isEmpty() ){
-
-						m_options = std::forward< E >( e ) ;
-					}else{
-						m_options += m_separator + std::forward< E >( e ) ;
-					}
+					m_options.append( e ) ;
+				}
+				void add( fuseOptions&& e )
+				{
+					_add( e ) ;
+				}
+				void add( fuseOptions& e )
+				{
+					_add( e ) ;
 				}
 				template< typename E,typename ... T >
 				void add( E&& e,T&& ... m )
@@ -644,71 +697,75 @@ public:
 					this->add( std::forward< E >( e ) ) ;
 					this->add( std::forward< T >( m ) ... ) ;
 				}
-				const QString& get() const
+				template< typename E >
+				void addFront( E&& e )
+				{
+					m_options.insert( 0,std::forward< E >( e ) ) ;
+				}
+				bool doesNotContain( const QString& key ) const
+				{
+					return !this->contains( key ) ;
+				}
+				bool contains( const QString& key ) const
+				{
+					for( const auto& it : m_options ){
+
+						if( it.contains( key,Qt::CaseInsensitive ) ){
+
+							return true ;
+						}
+					}
+
+					return false ;
+				}
+				const QStringList& get() const
 				{
 					return m_options ;
 				}
 				QString extractStartsWith( const QString& e )
 				{
-					auto tmp = m_options ;
-					m_options.clear() ;
+					for( int i = 0 ; i < m_options.size() ; i++ ){
 
-					QString m ;
+						if( m_options[ i ].startsWith( e ) ){
 
-					auto s = m_separator[ 0 ].toLatin1() ;
-
-					for( const auto& it : utility::split( tmp,s ) ){
-
-						if( it.startsWith( e ) ){
-
-							m = it ;
-						}else{
-							this->add( it ) ;
+							return m_options.takeAt( i ) ;
 						}
 					}
 
-					return m ;
+					return {} ;
 				}
 			private:
-				QString& m_options ;
-				QString m_separator ;
+				void _add( const fuseOptions& s ) ;
+
+				QStringList& m_options ;
 			};
 
 			class fuseOptions : public Options
 			{
 			public:
-				fuseOptions( QString& m ) : Options( m,"," )
+				fuseOptions( QStringList& m ) : Options( m )
 				{
 				}
-				void addPair( const QString& key,const QString& value,
-					      separator s = separator::equal_sign )
-				{
-					Options::addPair( key,value,s ) ;
-				}
 			} ;
 
 			class exeOptions : public Options
 			{
 			public:
-				exeOptions( QString& m ) : Options( m," " )
+				exeOptions( QStringList& m ) : Options( m )
 				{
 				}
-				void addPair( const QString& key,const QString& value,
-					      separator s = separator::space )
-				{
-					Options::addPair( key,value,s ) ;
-				}
 			} ;
 
-			commandOptions( const engines::engine::cmdArgsList& e,
-					const QString& f,
-					const QString& g = QString() ) ;
+			commandOptions() ;
+			commandOptions( bool creating,
+			                const engines::engine& engine,
+			                const engines::engine::cmdArgsList& e ) ;
 
-			const QString& constExeOptions() const
+			const QStringList& constExeOptions() const
 			{
 				return m_exeOptions ;
 			}
-			const QString& constFuseOpts() const
+			const QStringList& constFuseOpts() const
 			{
 				return m_fuseOptions ;
 			}
@@ -729,8 +786,8 @@ public:
 				return m_fuseOptions ;
 			}
 		private:
-			QString m_exeOptions ;
-			QString m_fuseOptions ;
+			QStringList m_exeOptions ;
+			QStringList m_fuseOptions ;
 			QString m_subtype ;
 			QString m_mode ;
 		};
@@ -744,7 +801,7 @@ public:
 	engines() ;
 	static const engines& instance() ;
 	bool atLeastOneDealsWithFiles() const ;
-	QStringList mountInfo( const QStringList& ) const ;
+	volumeInfo::List mountInfo( const volumeInfo::List& ) const ;
 	QStringList enginesWithNoConfigFile() const ;
 	QStringList enginesWithConfigFile() const ;
 	const std::vector< engines::engine::Wrapper >& supportedEngines() const ;
@@ -784,7 +841,7 @@ public:
 	                                     const QString& configPath = QString() ) const ;
 	const engine& getUnKnown() const ;
 	const engine& getByName( const QString& e ) const ;
-	const engine& getByFuseName( const QString& e ) const ;
+	const engine& getByFsName( const QString& e ) const ;
 private:
 	std::vector< std::unique_ptr< engines::engine > > m_backends ;
 	std::vector< engines::engine::Wrapper > m_backendWrappers ;
