@@ -17,6 +17,10 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+
+#ifndef CUSTOM_BACKEND_H
+#define CUSTOM_BACKEND_H
+
 #include "../engines.h"
 
 #include <QStringList>
@@ -27,24 +31,24 @@
 class custom : public engines::engine
 {
 public:
+	static engines::engine::args set_command( const engines::engine& engine,
+						  const QByteArray& password,
+						  const engines::engine::cmdArgsList& args,
+						  bool create ) ;
+
 	static void addEngines( std::vector< std::unique_ptr< engines::engine > >& ) ;
 
-	struct opts{
-		engines::engine::BaseOptions baseOpts ;
-		QString mountControlStructure ;
-		QString createControlStructure ;
-	};
-
-	custom( custom::opts ) ;
+	custom( engines::engine::BaseOptions baseOpts ) ;
 
 	engines::engine::status errorCode( const QString& e,int s ) const override ;
 
 	engines::engine::args command( const QByteArray& password,
-				       const engines::engine::cmdArgsList& args ) const override ;
+				       const engines::engine::cmdArgsList& args,
+				       bool create ) const override ;
 
 	void GUICreateOptions( const engines::engine::createGUIOptions& ) const override ;
 private:
-	const QString m_mountControlStructure ;
-	const QString m_createControlStructure ;
 	engines::version m_version ;
 } ;
+
+#endif
