@@ -24,12 +24,13 @@ class gocryptfs : public engines::engine
 public:
 	gocryptfs() ;
 
-	void updateOptions( engines::engine::cmdArgsList::options&,bool ) const override ;
+	void updateOptions( engines::engine::cmdArgsList&,bool ) const override ;
 
 	engines::engine::status errorCode( const QString& e,int s ) const override ;
 
 	engines::engine::args command( const QByteArray& password,
-				       const engines::engine::cmdArgsList& args ) const override ;
+				       const engines::engine::cmdArgsList& args,
+				       bool create ) const override ;
 
 	void GUICreateOptions( const engines::engine::createGUIOptions& ) const override ;
 
