@@ -21,6 +21,8 @@
 #include "../settings.h"
 #include "options.h"
 
+#include "custom.h"
+
 static engines::engine::BaseOptions _setOptions()
 {
 	engines::engine::BaseOptions s ;
@@ -31,25 +33,29 @@ static engines::engine::BaseOptions _setOptions()
 	 * enough for sshfs.
 	 */
 
-	s.passwordFormat        = "%{password}\n%{password}\n%{password}\n%{password}\n%{password}\n%{password}\n%{password}\n%{password}\n%{password}\n%{password}" ;
-
+	s.passwordFormat              = "%{password}\n%{password}\n%{password}\n%{password}\n%{password}\n%{password}\n%{password}\n%{password}\n%{password}\n%{password}" ;
 	s.backendTimeout              = settings::instance().sshfsBackendTimeout() ;
 	s.takesTooLongToUnlock        = false ;
 	s.supportsMountPathsOnWindows = true ;
 	s.autorefreshOnMountUnMount   = true ;
 	s.backendRequireMountPath     = true ;
+	s.backendRunsInBackGround     = true ;
+	s.usesOnlyMountPoint          = false ;
+	s.likeSsh               = true ;
 	s.requiresPolkit        = false ;
 	s.customBackend         = false ;
-	s.requiresAPassword     = false ;
+	s.requiresAPassword     = true ;
 	s.hasConfigFile         = false ;
 	s.autoMountsOnCreate    = true ;
 	s.hasGUICreateOptions   = false ;
 	s.setsCipherPath        = true ;
-	s.windowsUnMountCommand = "" ;
+	s.acceptsSubType        = true ;
+	s.acceptsVolName        = true ;
 	s.executableName        = "sshfs" ;
 	s.releaseURL            = "https://api.github.com/repos/libfuse/sshfs/releases" ;
 	s.windowsInstallPathRegistryKey = "SOFTWARE\\SSHFS-Win" ;
 	s.windowsInstallPathRegistryValue = "InstallDir" ;
+	s.windowsUnMountCommand           = QStringList{ "taskkill","/F","/PID","%{PID}" } ;
 	s.failedToMountList     = QStringList{ "ssh:","read:","Cannot create WinFsp-FUSE file system" } ;
 	s.successfulMountedList = QStringList{ "has been started" } ;
 	s.fuseNames             = QStringList{ "fuse.sshfs" } ;
@@ -57,9 +63,19 @@ static engines::engine::BaseOptions _setOptions()
 	s.notFoundCode          = engines::engine::status::sshfsNotFound ;
 	s.versionInfo           = { { "--version",true,2,0 } } ;
 
+	s.mountControlStructure = "%{mountOptions} %{cipherFolder} %{mountPoint} %{fuseOpts}" ;
+
 	if( utility::platformIsWindows() ){
 
-		s.minimumVersion = "3.5.2" ;
+		s.autoCreatesMountPoint = true ;
+		s.autoDeletesMountPoint = true ;
+
+		s.versionMinimum = "3.5.2" ;
+		s.sshOptions = "create_file_umask=0000,create_dir_umask=0000,umask=0000,idmap=user,StrictHostKeyChecking=no" ;
+	}else{
+		s.autoCreatesMountPoint = false ;
+		s.autoDeletesMountPoint = false ;
+		s.sshOptions = "idmap=user,StrictHostKeyChecking=no" ;
 	}
 
 	return s ;
@@ -68,11 +84,22 @@ static engines::engine::BaseOptions _setOptions()
 sshfs::sshfs() :
 	engines::engine( _setOptions() ),
 	m_environment( engines::engine::getProcessEnvironment() ),
-	m_version_greater_or_equal_minimum( false,*this,this->minimumVersion() )
+	m_version_greater_or_equal_minimum( false,*this,this->minimumVersion() ),
+	m_sshAuthSock( qgetenv( "SSH_AUTH_SOCK" ) )
 {
 }
 
-engines::engine::status sshfs::passAllRequirenments( const engines::engine::cmdArgsList::options& opt ) const
+bool sshfs::requiresAPassword( const engines::engine::cmdArgsList& e ) const
+{
+	if( !m_sshAuthSock.isEmpty() || !e.identityFile.isEmpty() || !e.identityAgent.isEmpty() ){
+
+		return false ;
+	}else{
+		return engines::engine::requiresAPassword( e ) ;
+	}
+}
+
+engines::engine::status sshfs::passAllRequirenments( const engines::engine::cmdArgsList& opt ) const
 {
 	if( utility::platformIsWindows() ){
 
@@ -89,7 +116,7 @@ engines::engine::status sshfs::passAllRequirenments( const engines::engine::cmdA
 			return engines::engine::status::backEndFailedToMeetMinimumRequirenment ;
 		}
 	}else{
-		return engines::engine::status::success ;
+		return engines::engine::passAllRequirenments( opt ) ;
 	}
 }
 
@@ -98,82 +125,88 @@ const QProcessEnvironment& sshfs::getProcessEnvironment() const
 	return m_environment ;
 }
 
-engines::engine::args sshfs::command( const QByteArray& password,
-				      const engines::engine::cmdArgsList& args ) const
+void sshfs::updateOptions( engines::engine::commandOptions& opts,
+			   const engines::engine::cmdArgsList& args,
+			   bool creating ) const
 {
-	Q_UNUSED( password )
+	Q_UNUSED( creating )
 
-	engines::engine::commandOptions m( args,this->name(),this->name() ) ;
-
-	auto fuseOptions = m.fuseOpts() ;
-	auto exeOptions  = m.exeOptions() ;
-
-	if( !args.opt.key.isEmpty() ){
-
-		fuseOptions.add( "password_stdin" ) ;
-	}
+	auto fuseOptions = opts.fuseOpts() ;
+	auto exeOptions  = opts.exeOptions() ;
 
 	if( fuseOptions.doesNotContain( "StrictHostKeyChecking=" ) ){
 
-		fuseOptions.addPair( "StrictHostKeyChecking","no" ) ;
+		fuseOptions.add( "StrictHostKeyChecking=no" ) ;
 	}
 
 	if( utility::platformIsWindows() ){
 
 		exeOptions.add( "-f" ) ;
 
-		auto m = utility::removeFirstAndLast( args.mountPoint,1,1 ) ;
+		if( utility::isDriveLetter( args.mountPoint ) ){
 
-		if( utility::isDriveLetter( m ) ){
+			if( !exeOptions.contains( "--VolumePrefix=" ) &&
+					args.boolOptions.unlockInReverseMode ){
 
-			if( !fuseOptions.contains( "--VolumePrefix=" ) ){
-
-				auto s = fuseOptions.extractStartsWith( "UseNetworkDrive=" ) ;
-
-				if( utility::endsWithAtLeastOne( s,"yes","Yes","YES" ) ){
-
-					auto x = args.cipherFolder ;
-					x.replace( ":",";" ) ;
-					exeOptions.add ( "--VolumePrefix=\\mysshfs\\" + x ) ;
-				}
+				auto x = args.cipherFolder ;
+				x.replace( ":",";" ) ;
+				exeOptions.add( "--VolumePrefix=\\mysshfs\\" + x ) ;
 			}
-		}else{
-			/*
-			 * A user is trying to use a folder as a mount path and sshfs
-			 * requires the mount path to not exist and we are deleting
-			 * it because SiriKali created it previously.
-			 */
-			utility::removeFolder( m,5 ) ;
 		}
 	}
 
-	if( fuseOptions.contains( "IdentityAgent" ) ){
+	if( args.key.isEmpty() ){
 
-		auto m = "IdentityAgent=" ;
+		fuseOptions.add( "PreferredAuthentications=publickey" ) ;
 
-		auto n = fuseOptions.extractStartsWith( m ).replace( m,"" ) ;
+		auto _set_identity_agent = [ & ](){
 
-		m_environment.insert( "SSH_AUTH_SOCK",n ) ;
+			if( args.identityAgent.isEmpty() ){
+
+				if( m_sshAuthSock.isEmpty() ){
+
+					utility::debug() << "Sshfs: identityAgent Not Set" ;
+					m_environment.remove( "SSH_AUTH_SOCK" ) ;
+				}else{
+					utility::debug() << "Sshfs: From Env, Setting Env Variable Of: SSH_AUTH_SOCK=" + m_sshAuthSock ;
+
+					m_environment.insert( "SSH_AUTH_SOCK",m_sshAuthSock ) ;
+				}
+			}else{
+				utility::debug() << "Sshfs: Setting Env Variable Of: SSH_AUTH_SOCK=" + args.identityAgent ;
+				m_environment.insert( "SSH_AUTH_SOCK",args.identityAgent ) ;
+			}
+		} ;
+
+		if( args.identityFile.isEmpty() ){
+
+			_set_identity_agent() ;
+		}else{
+			_set_identity_agent() ;
+
+			fuseOptions.add( "IdentityFile=" + args.identityFile ) ;
+		}
 	}else{
-		auto m = qgetenv( "SSH_AUTH_SOCK" ) ;
-
-		if( m.isEmpty() ){
-
-			m_environment.remove( "SSH_AUTH_SOCK" ) ;
-		}else{
-			m_environment.insert( "SSH_AUTH_SOCK",m ) ;
-		}
+		fuseOptions.add( "PreferredAuthentications=password" ) ;
+		fuseOptions.add( "password_stdin" ) ;
 	}
+}
 
-	QString s = "%1 %2 %3 %4 %5" ;
+engines::engine::args sshfs::command( const QByteArray& password,
+				      const engines::engine::cmdArgsList& args,
+				      bool create ) const
+{
+	return custom::set_command( *this,password,args,create ) ;
+}
 
-	auto cmd = s.arg( args.exe,
-			  exeOptions.get(),
-			  args.cipherFolder,
-			  args.mountPoint,
-			  fuseOptions.get() ) ;
+engines::engine::error sshfs::errorCode( const QString& e ) const
+{
+	if( e.contains( "invalid argument" ) ){
 
-	return { args,m,cmd } ;
+		return engines::engine::error::Failed ;
+	}else{
+		return engines::engine::errorCode( e ) ;
+	}
 }
 
 engines::engine::status sshfs::errorCode( const QString& e,int s ) const
