@@ -21,15 +21,16 @@
 #define FAVORITES_H
 
 #include <QString>
+
 #include <vector>
+#include <forward_list>
+
 #include "utility.h"
 #include "json_parser.hpp"
 
 class favorites
 {
 public:
-	enum class type{ sshfs,others } ;
-
 	class triState{
 	public:
 		triState() : m_state( triState::STATES::UNDEFINED )
@@ -66,6 +67,10 @@ public:
 		{
 			return !this->undefined() ;
 		}
+		operator bool() const
+		{
+			return this->True() ;
+		}
 		bool True() const
 		{
 			return m_state == STATES::TRUE ;
@@ -108,18 +113,28 @@ public:
 	struct entry
 	{
 		entry() ;
-		entry( const QString& volumePath ) ;
+		entry( const QString& volumePath,const QString& mountPath = QString() ) ;
+
+		bool hasValue() const
+		{
+			return !volumePath.isEmpty() ;
+		}
 
 		QString volumePath ;
 		QString mountPointPath ;
 		QString configFilePath ;
 		QString keyFile ;
 		QString idleTimeOut ;
+		QString identityFile ;
+		QString identityAgent ;
 		QString mountOptions ;
 		QString preMountCommand ;
 		QString postMountCommand ;
 		QString preUnmountCommand ;
 		QString postUnmountCommand ;
+		QString internalConfigPath ;
+
+		QByteArray password ;
 
 		bool reverseMode = false ;
 		bool volumeNeedNoPassword = false ;
@@ -134,7 +149,57 @@ public:
 		return m ;
 	}
 
-	using volumeList = std::vector< std::pair< favorites::entry,QByteArray > > ;
+	class volEntry{
+	public:
+		template< typename E,typename P >
+		volEntry( E&& e,P&& p,bool manage ) :
+			m_favorite( this->entry( std::forward< E >( e ),manage ) ),
+			m_password( std::forward< P >( p ) )
+		{
+		}
+		template< typename E >
+		volEntry( E&& e,bool manage ) :
+			m_favorite( this->entry( std::forward< E >( e ),manage ) ),
+			m_password( m_favorite.password )
+		{
+		}
+		volEntry( const favorites::entry& e ) :
+			m_favorite( e ),m_password( e.password )
+		{
+		}
+		template< typename P >
+		volEntry( const favorites::entry& e,P&& s ) :
+			m_favorite( e ),m_password( std::forward< P >( s ) )
+		{
+		}
+		const favorites::entry& favorite() const
+		{
+			return m_favorite ;
+		}
+		const QByteArray& password() const
+		{
+			return m_password ;
+		}
+		template< typename T >
+		void setPassword( T&& e )
+		{
+			m_password = std::forward< T >( e ) ;
+		}
+	private:
+		template< typename E >
+		const favorites::entry& entry( E&& e,bool manage )
+		{
+			Q_UNUSED( manage )
+			utility::debug() << "favorites managing temporary entry: " + e.volumePath ;
+			m_tmpFavorite = std::make_unique< favorites::entry >( std::forward< E >( e ) ) ;
+			return *m_tmpFavorite ;
+		}
+		std::unique_ptr< favorites::entry > m_tmpFavorite ;
+		const favorites::entry& m_favorite ;
+		QByteArray m_password ;
+	};
+
+	using volumeList = std::vector< volEntry > ;
 
 	enum class error{ ENTRY_ALREADY_EXISTS,FAILED_TO_CREATE_ENTRY,SUCCESS } ;
 
@@ -143,14 +208,24 @@ public:
 	void replaceFavorite( const favorites::entry&, const favorites::entry& ) ;
 	void removeFavoriteEntry( const favorites::entry& ) ;
 
-	std::vector< favorites::entry > readFavorites() const ;
+	const std::vector< favorites::entry >& readFavorites() const ;
 
-	utility2::result< favorites::entry > readFavorite( const QString&,const QString& = QString() ) const ;
+	volumeList readVolumeList() const ;
 
-	utility2::result< favorites::entry > readFavoriteByPath( const QString& ) const ;
+	const favorites::entry& readFavoriteByPath( const QString& configPath ) const ;
 
+	favorites::entry readFavoriteByFileSystemPath( const QString& configPath ) const ;
 
-	void updateFavorites() ;
+	const favorites::entry& unknown() const ;
+
+	const favorites::entry& readFavorite( const QString& volumePath,
+					      const QString& mountPath = QString() ) const ;
+
+	favorites() ;
+private:
+	void reload() ;
+	std::vector< favorites::entry > m_favorites ;
+	favorites::entry m_empty ;
 };
 
 #endif // FAVORITES_H
